@@ -1,20 +1,20 @@
 #! /usr/bin/env/python
 # -*- coding: iso-8859-1 -*-
 
-## Copyright 1999-2004 by LivingLogic AG, Bayreuth, Germany.
-## Copyright 1999-2004 by Walter Drwald
+## Copyright 1999-2005 by LivingLogic AG, Bayreuth/Germany.
+## Copyright 1999-2005 by Walter Drwald
 ##
 ## All Rights Reserved
 ##
 ## See xist/__init__.py for the license
 
-import sys, unittest, cStringIO, warnings, re
+import sys, unittest, cStringIO, warnings, re, cPickle
 
 from xml.sax import saxlib
 from xml.parsers import expat
 
 from ll import url
-from ll.xist import xsc, parsers, cssparsers, presenters, converters, helpers, errors, options, sims, xnd, xfind
+from ll.xist import xsc, parsers, cssparsers, presenters, converters, helpers, options, sims, xnd, xfind
 from ll.xist.ns import wml, ihtml, html, chars, css, abbr, specials, htmlspecials, php, xml, tld
 
 
@@ -27,6 +27,137 @@ options.reprtab = "  "
 restrictedchars = re.compile(u"[\x00-\x08\x0B-\x0C\x0E-\x1F\x7F-\x84\x86-\x9F]")
 
 
+def createattr():
+	return html.span.Attrs.lang(
+		True,
+		False,
+		url.URL("http://www.python.org/"),
+		html.abbr(
+			xml.XML10(),
+			"hurz",
+			specials.tab(),
+			abbr.xist(),
+			None,
+			1,
+			2.0,
+			"3",
+			u"4",
+			(5, 6),
+			[7, 8],
+			html.span("gurk"),
+			title="hurz"
+		)
+	)
+
+
+def createattrs():
+	return html.span.Attrs(
+		lang=(
+			True,
+			False,
+			url.URL("http://www.python.org/"),
+			html.abbr(
+				xml.XML10(),
+				"hurz",
+				specials.tab(),
+				abbr.xist(),
+				None,
+				1,
+				2.0,
+				"3",
+				u"4",
+				(5, 6),
+				[7, 8],
+				html.span("gurk"),
+				title="hurz"
+			)
+		)
+	)
+
+
+def createelement():
+	return html.span(
+		1,
+		2,
+		class_="gurk",
+		id=(1, 2, (3, 4)),
+		lang=(
+			True,
+			False,
+			url.URL("http://www.python.org/"),
+			html.abbr(
+				xml.XML10(),
+				"hurz",
+				specials.tab(),
+				abbr.xist(),
+				None,
+				1,
+				2.0,
+				"3",
+				u"4",
+				(5, 6),
+				[7, 8],
+				html.span("gurk"),
+				title="hurz"
+			)
+		)
+	)
+
+
+def createfrag():
+	return xsc.Frag(
+		xml.XML10(),
+		html.DocTypeHTML401transitional(),
+		xsc.Comment("gurk"),
+		"hurz",
+		specials.tab(),
+		abbr.xist(),
+		None,
+		True,
+		False,
+		1,
+		2.0,
+		"3",
+		u"4",
+		(5, 6),
+		[7, 8],
+		html.div(
+			align="left"
+		),
+		url.URL("http://www.python.org/"),
+		html.span(
+			1,
+			2,
+			class_="gurk",
+			id=(1, 2, (3, 4)),
+			lang=(
+				True,
+				False,
+				url.URL("http://www.python.org/"),
+				html.abbr(
+					xml.XML10(),
+					"hurz",
+					specials.tab(),
+					abbr.xist(),
+					None,
+					1,
+					2.0,
+					"3",
+					u"4",
+					(5, 6),
+					[7, 8],
+					html.span("gurk"),
+					title="hurz"
+				)
+			)
+		)
+	)
+
+
+def allnodes():
+	return (xsc.Null, createattr(), createattrs(), createelement(), createfrag())
+
+
 class XISTTest(unittest.TestCase):
 	def check_lenunicode(self, node, _len, content):
 		self.assertEqual(len(node), _len)
@@ -155,134 +286,8 @@ class XISTTest(unittest.TestCase):
 			self.check_lenunicode(cls(1, (2, 3)), 3, u"123")
 			self.check_lenunicode(cls(1, (None, None)), 1, u"1")
 
-	def createattr(self):
-		return html.span.Attrs.lang(
-			True,
-			False,
-			url.URL("http://www.python.org/"),
-			html.abbr(
-				xml.XML10(),
-				"hurz",
-				specials.tab(),
-				abbr.xist(),
-				None,
-				1,
-				2.0,
-				"3",
-				u"4",
-				(5, 6),
-				[7, 8],
-				html.span("gurk"),
-				title="hurz"
-			)
-		)
-
-	def createattrs(self):
-		return html.span.Attrs(
-			lang=(
-				True,
-				False,
-				url.URL("http://www.python.org/"),
-				html.abbr(
-					xml.XML10(),
-					"hurz",
-					specials.tab(),
-					abbr.xist(),
-					None,
-					1,
-					2.0,
-					"3",
-					u"4",
-					(5, 6),
-					[7, 8],
-					html.span("gurk"),
-					title="hurz"
-				)
-			)
-		)
-
-	def createelement(self):
-		return html.span(
-			1,
-			2,
-			class_="gurk",
-			id=(1, 2, (3, 4)),
-			lang=(
-				True,
-				False,
-				url.URL("http://www.python.org/"),
-				html.abbr(
-					xml.XML10(),
-					"hurz",
-					specials.tab(),
-					abbr.xist(),
-					None,
-					1,
-					2.0,
-					"3",
-					u"4",
-					(5, 6),
-					[7, 8],
-					html.span("gurk"),
-					title="hurz"
-				)
-			)
-		)
-
-	def createfrag(self):
-		return xsc.Frag(
-			xml.XML10(),
-			html.DocTypeHTML401transitional(),
-			xsc.Comment("gurk"),
-			"hurz",
-			specials.tab(),
-			abbr.xist(),
-			None,
-			True,
-			False,
-			1,
-			2.0,
-			"3",
-			u"4",
-			(5, 6),
-			[7, 8],
-			html.div(
-				align="left"
-			),
-			url.URL("http://www.python.org/"),
-			html.span(
-				1,
-				2,
-				class_="gurk",
-				id=(1, 2, (3, 4)),
-				lang=(
-					True,
-					False,
-					url.URL("http://www.python.org/"),
-					html.abbr(
-						xml.XML10(),
-						"hurz",
-						specials.tab(),
-						abbr.xist(),
-						None,
-						1,
-						2.0,
-						"3",
-						u"4",
-						(5, 6),
-						[7, 8],
-						html.span("gurk"),
-						title="hurz"
-					)
-				)
-			)
-		)
-
-	def allnodes(self):
-		return (xsc.Null, self.createattr(), self.createattrs(), self.createelement(), self.createfrag())
-
 	def test_standardmethods(self):
-		for node in self.allnodes():
+		for node in allnodes():
 			node.compact()
 			node.normalized()
 			list(node.walk((True, xsc.enterattrs, xsc.entercontent)))
@@ -293,20 +298,20 @@ class XISTTest(unittest.TestCase):
 			node.normalized().compact().pretty()
 
 	def test_standardmethods2(self):
-		for node in (self.createelement(), self.createfrag()):
+		for node in (createelement(), createfrag()):
 			node.sorted()
 			node.shuffled()
 			node.reversed()
 
 	def test_stringify(self):
-		for node in self.allnodes():
+		for node in allnodes():
 			unicode(node)
 			str(node)
 			node.asString()
 			node.asBytes()
 
 	def test_asText(self):
-		for node in self.allnodes():
+		for node in allnodes():
 			node.asText()
 			node.asText(monochrome=True)
 			node.asText(squeezeBlankLines=True)
@@ -343,12 +348,13 @@ class XISTTest(unittest.TestCase):
 		self.assertEqual(node*3, xsc.Frag(html.div(), html.div(), html.div()))
 
 	def test_text(self):
-		node = xsc.Text("test")
+		s = "test"
+		node = xsc.Text(s)
 		hash(node)
 		self.assertEqual(len(node), 4)
 		self.assertEqual(node[1], xsc.Text("e"))
-		self.assertEqual(3*node, xsc.Text(3*node.content))
-		self.assertEqual(node*3, xsc.Text(node.content*3))
+		self.assertEqual(3*node, xsc.Text(3*s))
+		self.assertEqual(node*3, xsc.Text(s*3))
 		self.assertEqual(node[1:3], xsc.Text("es"))
 		self.assertEqual(node.capitalize(), xsc.Text("Test"))
 		self.assertEqual(node.center(8), xsc.Text("  test  "))
@@ -366,10 +372,12 @@ class XISTTest(unittest.TestCase):
 		self.assertEqual(node.isupper(), False)
 		self.assertEqual(node.join(xsc.Frag(list("abc"))), xsc.Frag("a", "test", "b", "test", "c"))
 		self.assertEqual(node.ljust(6), xsc.Text("test  "))
+		self.assertEqual(node.ljust(6, "."), xsc.Text("test.."))
 		self.assertEqual(node.lower(), xsc.Text("test"))
 		self.assertEqual(xsc.Text("  test").lstrip(), xsc.Text("test"))
 		self.assertEqual(node.replace("s", "x"), xsc.Text("text"))
 		self.assertEqual(node.rjust(6), xsc.Text("  test"))
+		self.assertEqual(node.rjust(6, "."), xsc.Text("..test"))
 		self.assertEqual(xsc.Text("test  ").rstrip(), xsc.Text("test"))
 		self.assertEqual(node.rfind("s"), 2)
 		self.assertEqual(node.rindex("s"), 2)
@@ -381,13 +389,55 @@ class XISTTest(unittest.TestCase):
 		self.assertEqual(node.title(), xsc.Text("Test"))
 		self.assertEqual(node.upper(), xsc.Text("TEST"))
 
+	def test_charref(self):
+		node = chars.ouml()
+		hash(node)
+		self.assertEqual(len(node), 1)
+		self.assertEqual(node[0], xsc.Text(u""))
+		self.assertEqual(3*node, xsc.Text(u""))
+		self.assertEqual(node*3, xsc.Text(u""))
+		self.assertEqual(node[1:-2], xsc.Text())
+		self.assertEqual(node.capitalize(), xsc.Text(u""))
+		self.assertEqual(node.center(5), xsc.Text(u"    "))
+		self.assertEqual(node.count(u"t"), 0)
+		self.assertEqual(node.endswith(u""), True)
+		self.assertEqual(node.index(u""), 0)
+		self.assertEqual(node.isalpha(), True)
+		self.assertEqual(node.isalnum(), True)
+		self.assertEqual(node.isdecimal(), False)
+		self.assertEqual(node.isdigit(), False)
+		self.assertEqual(node.islower(), True)
+		self.assertEqual(node.isnumeric(), False)
+		self.assertEqual(node.isspace(), False)
+		self.assertEqual(node.istitle(), False)
+		self.assertEqual(node.isupper(), False)
+		self.assertEqual(node.ljust(3), xsc.Text(u"  "))
+		self.assertEqual(node.ljust(3, "."), xsc.Text(u".."))
+		self.assertEqual(node.lower(), xsc.Text(u""))
+		self.assertEqual(node.replace(u"", "x"), xsc.Text("x"))
+		self.assertEqual(node.rjust(3), xsc.Text(u"  "))
+		self.assertEqual(node.rjust(3, "."), xsc.Text(u".."))
+		self.assertEqual(node.rfind(u""), 0)
+		self.assertEqual(node.rindex(u""), 0)
+		self.assertEqual(node.startswith(u""), True)
+		self.assertEqual(node.swapcase(), xsc.Text(u""))
+		self.assertEqual(node.title(), xsc.Text(u""))
+		self.assertEqual(node.upper(), xsc.Text(u""))
+
 	def test_getsetitem(self):
 		for cls in (xsc.Frag, html.div):
 			for attr in ("class_", (xml, "lang")):
-				node = cls(html.div(html.div({attr: "gurk"})))
-				self.assertEqual(str(node[[0, 0, attr]]), "gurk")
-				node[[0, 0, attr]] = "hurz"
-				self.assertEqual(str(node[[0, 0, attr]]), "hurz")
+				node = cls(html.div("foo", html.div({attr: "gurk"}), "bar"))
+				self.assert_(node[[]] is node)
+				self.assertEqual(str(node[[0, 1, attr]]), "gurk")
+				node[[0, 1, attr]] = "hurz"
+				self.assertEqual(str(node[[0, 1, attr]]), "hurz")
+				i = node[0][xsc.Text]
+				self.assertEqual(str(i.next()), "foo")
+				self.assertEqual(str(i.next()), "bar")
+				self.assertRaises(StopIteration, i.next)
+				self.assertRaises(ValueError, node.__setitem__, [], None)
+				self.assertRaises(ValueError, node.__delitem__, [])
 
 	def mappedmapper(self, node, converter):
 		if isinstance(node, xsc.Text):
@@ -395,88 +445,27 @@ class XISTTest(unittest.TestCase):
 		return node
 
 	def test_conv(self):
-		node = self.createfrag()
+		node = createfrag()
 		node.conv()
 		node.conv(converters.Converter())
 		node.mapped(self.mappedmapper, converters.Converter())
 
 	def test_repr(self):
-		for node in self.allnodes():
+		tests = allnodes()
+		allpresenters = [c for c in presenters.__dict__.itervalues() if isinstance(c, type) and c is not presenters.Presenter and issubclass(c, presenters.Presenter)]
+		for node in tests:
 			repr(node)
-			for class_ in presenters.__dict__.itervalues():
-				if isinstance(class_, type) and issubclass(class_, presenters.Presenter):
-					node.repr(class_())
-			for showLocation in (False, True):
-				for showPath in (False, True):
-					node.repr(presenters.TreePresenter(showLocation=showLocation, showPath=showPath))
-
-	def node2str(self, node):
-		if isinstance(node, xsc.Node):
-			if isinstance(node, xsc.Text):
-				return "#"
-			else:
-				return node.xmlname[True]
-		else:
-			return ".".join(map(self.node2str, node))
-
-	def check_walk(self, node, filter, result, filterpath=False, walkpath=False):
-		self.assertEqual(map(self.node2str, node.walk(filter, filterpath=filterpath, walkpath=walkpath)), result)
-
-	def test_walk1(self):
-		node = self.createfrag()
-		def filter1(node):
-			return (True, xsc.enterattrs, xsc.entercontent, True)
-		def filter2(path):
-			return (True, xsc.enterattrs, xsc.entercontent, True)
-
-		list(node.walk((True, xsc.entercontent, True)))
-		list(node.walk((True, xsc.entercontent, True), walkpath=True))
-		list(node.walk(filter1))
-		list(node.walk(filter1, walkpath=True))
-		list(node.walk(filter2, filterpath=True))
-		list(node.walk(filter2, filterpath=True, walkpath=True))
-
-	def test_walk2(self):
-		node = html.div(html.tr(html.th("gurk"), html.td("hurz"), id=html.b(42)), class_=html.i("hinz"))
-
-		def filtertopdown(node):
-			return (isinstance(node, xsc.Element), xsc.entercontent)
-		def filterbottomup(node):
-			return (xsc.entercontent, isinstance(node, xsc.Element))
-		def filtertopdownattrs(node):
-			return (isinstance(node, xsc.Element), xsc.enterattrs, xsc.entercontent)
-		def filterbottomupattrs(node):
-			return (xsc.enterattrs, xsc.entercontent, isinstance(node, xsc.Element))
-		def filtertopdowntextonlyinattr(path):
-			for node in path:
-				if isinstance(node, xsc.Attr):
-					inattr = True
-					break
-			else:
-				inattr = False
-			node = path[-1]
-			if isinstance(node, xsc.Element):
-				return (True, xsc.enterattrs, xsc.entercontent)
-			if inattr and isinstance(node, xsc.Text):
-				return (True, )
-			else:
-				return (xsc.entercontent, )
-
-		def filtertopdownattrwithoutcontent(node):
-			if isinstance(node, xsc.Element):
-				return (True, xsc.entercontent, xsc.enterattrs)
-			elif isinstance(node, (xsc.Attr, xsc.Text)):
-				return (True, )
-			else:
-				return (xsc.entercontent, )
-
-		self.check_walk(node, filtertopdown, ["div", "tr", "th", "td"])
-		self.check_walk(node, filterbottomup, ["th", "td", "tr", "div"])
-		self.check_walk(node, filtertopdownattrs, ["div", "i", "tr", "b", "th", "td"])
-		self.check_walk(node, filtertopdownattrs, ["div", "div.class.i", "div.tr", "div.tr.id.b", "div.tr.th", "div.tr.td"], walkpath=True)
-		self.check_walk(node, filterbottomupattrs, ["div.class.i", "div.tr.id.b", "div.tr.th", "div.tr.td", "div.tr", "div"], walkpath=True)
-		self.check_walk(node, filtertopdowntextonlyinattr, ["div", "div.class.i", "div.class.i.#", "div.tr", "div.tr.id.b", "div.tr.id.b.#", "div.tr.th", "div.tr.td"], filterpath=True, walkpath=True)
-		self.check_walk(node, filtertopdownattrwithoutcontent, ["div", "div.tr", "div.tr.th", "div.tr.th.#", "div.tr.td", "div.tr.td.#", "div.tr.id", "div.class"], walkpath=True)
+			for class_ in allpresenters:
+				presenter = class_()
+				# do it multiple time, to make sure the presenter gets properly reset
+				for i in xrange(3):
+					node.repr(presenter)
+			for showlocation in (False, True):
+				for showpath in (False, True):
+					presenter = presenters.TreePresenter(showlocation=showlocation, showpath=showpath)
+					# do it multiple time, to make sure the presenter gets properly reset
+					for i in xrange(3):
+						node.repr(presenter)
 
 	def test_locationeq(self):
 		l1 = xsc.Location(sysid="gurk", pubid="http://gurk.com", line=42, col=666)
@@ -617,25 +606,25 @@ class XISTTest(unittest.TestCase):
 		)
 		keys = node.attrs.keys()
 		keys.sort()
-		keys.remove("lang")
+		keys.remove("class_")
 
-		keys1 = node.attrs.without(["lang"]).keys()
+		keys1 = node.attrs.without(["class_"]).keys()
 		keys1.sort()
 		self.assertEqual(keys, keys1)
 
 		keys.remove((xml2, "space"))
-		keys2 = node.attrs.without(["lang", (xml, "space")]).keys()
+		keys2 = node.attrs.without(["class_", (xml, "space")]).keys()
 		keys2.sort()
 		self.assertEqual(keys, keys2)
 
 		keys.remove((xml2, "lang"))
 		keys.remove((xml2, "base"))
-		keys3 = node.attrs.without(["lang"], [xml]).keys()
+		keys3 = node.attrs.without(["class_"], [xml]).keys()
 		keys3.sort()
 		self.assertEqual(keys, keys3)
 
 		# Check that non existing attrs are handled correctly
-		keys4 = node.attrs.without(["lang", "src"], keepglobals=False).keys()
+		keys4 = node.attrs.without(["class_", "src"], keepglobals=False).keys()
 		keys4.sort()
 		self.assertEqual(keys, keys4)
 
@@ -655,21 +644,21 @@ class XISTTest(unittest.TestCase):
 		keys.sort()
 		keys.remove("lang")
 
-		self.assertEquals(node.attrs.with([u"lang"]).keys(), [u"lang"])
+		self.assertEquals(node.attrs.with(["lang"]).keys(), ["lang"])
 
-		keys1 = node.attrs.with([u"lang", u"align"]).keys()
+		keys1 = node.attrs.with(["lang", "align"]).keys()
 		keys1.sort()
-		self.assertEqual(keys1, [u"align", u"lang"])
+		self.assertEqual(keys1, ["align", "lang"])
 
-		keys = [u"lang", (xml2, u"lang")]
+		keys = ["lang", (xml2, "lang")]
 		keys.sort()
 		keys2 = node.attrs.with(keys).keys()
 		keys2.sort()
 		self.assertEqual(keys2, keys)
 
-		keys = [u"lang", (xml2, u"lang"), (xml2, u"space")]
+		keys = ["lang", (xml2, "lang"), (xml2, "space")]
 		keys.sort()
-		keys3 = node.attrs.with([u"lang"], [xml]).keys()
+		keys3 = node.attrs.with(["lang"], [xml]).keys()
 		keys3.sort()
 		self.assertEqual(keys3, keys)
 
@@ -683,7 +672,7 @@ class XISTTest(unittest.TestCase):
 		node = Test()
 		self.assert_(node.attrs.has("withdef"))
 		self.assert_(not node.attrs.has("withoutdef"))
-		self.assertRaises(errors.IllegalAttrError, node.attrs.has, "illegal")
+		self.assertRaises(xsc.IllegalAttrError, node.attrs.has, "illegal")
 		node = Test(withdef=None)
 		self.assert_(not node.attrs.has("withdef"))
 
@@ -815,7 +804,7 @@ class XISTTest(unittest.TestCase):
 
 	def test_allowedattr(self):
 		self.assertEquals(html.a.Attrs.allowedattr("href"), html.a.Attrs.href)
-		self.assertRaises(errors.IllegalAttrError, html.a.Attrs.allowedattr, "gurk")
+		self.assertRaises(xsc.IllegalAttrError, html.a.Attrs.allowedattr, "gurk")
 		self.assertEquals(html.a.Attrs.allowedattr((xml, "lang")), xml.Attrs.lang)
 
 	def test_plaintableattrs(self):
@@ -874,7 +863,7 @@ class XISTTest(unittest.TestCase):
 		self.assert_("hurz" not in attrs)
 
 		# No global attributes inside global attributes
-		self.assertRaises(errors.IllegalAttrError, xml.Attrs, xml.Attrs(lang="de"))
+		self.assertRaises(xsc.IllegalAttrError, xml.Attrs, xml.Attrs(lang="de"))
 
 	def test_classrepr(self):
 		repr(xsc.Base)
@@ -1171,22 +1160,26 @@ class NamespaceTest(unittest.TestCase):
 		ns.updateexisting(*a)
 		self.assertEquals(ns.element("foo"), ns2.foo)
 		self.assertEquals(ns.element("bar"), ns2.bar)
-		self.assertRaises(errors.IllegalElementError, ns.element, "foo2")
-		self.assertRaises(errors.IllegalElementError, ns.element, "bar2")
+		self.assertRaises(xsc.IllegalElementError, ns.element, "foo2")
+		self.assertRaises(xsc.IllegalElementError, ns.element, "bar2")
 
 	def test_attributeexamples(self):
-		self.assertEqual(xsc.amp.xmlname, (u"amp", u"amp"))
+		self.assertEqual(xsc.amp.__name__, "amp")
+		self.assertEqual(xsc.amp.xmlname, u"amp")
 		self.assert_(xsc.amp.__ns__ is None)
 		self.assertEqual(xsc.amp.xmlprefix(), None)
 
-		self.assertEqual(chars.uuml.xmlname, (u"uuml", u"uuml"))
+		self.assertEqual(chars.uuml.__name__, "uuml")
+		self.assertEqual(chars.uuml.xmlname, u"uuml")
 		self.assert_(chars.uuml.__ns__ is chars)
 		self.assertEqual(chars.uuml.xmlprefix(), "chars")
 
-		self.assertEqual(html.a.Attrs.class_.xmlname, (u"class_", u"class"))
+		self.assertEqual(html.a.Attrs.class_.__name__, "class_")
+		self.assertEqual(html.a.Attrs.class_.xmlname, u"class")
 		self.assert_(html.a.Attrs.class_.__ns__ is None)
 
-		self.assertEqual(xml.Attrs.lang.xmlname, (u"lang", u"lang"))
+		self.assertEqual(xml.Attrs.lang.__name__, "lang")
+		self.assertEqual(xml.Attrs.lang.xmlname, u"lang")
 		self.assert_(xml.Attrs.lang.__ns__ is xml)
 		self.assertEqual(xml.Attrs.lang.xmlprefix(), "xml")
 
@@ -1472,7 +1465,7 @@ class ParseTest(unittest.TestCase):
 		node = parsers.parseString("<z>gurk&amp;hurz&#42;hinz&#x666;hunz</z>", saxparser=parsers.SGMLOPParser)
 		self.assertEqual(len(node), 1)
 		self.assertEqual(len(node[0]), 1)
-		self.assertEqual(node[0][0].startloc.getSystemId(), "root:STRING")
+		self.assertEqual(node[0][0].startloc.getSystemId(), "STRING")
 		self.assertEqual(node[0][0].startloc.getLineNumber(), 1)
 
 	def test_parselocationexpat(self):
@@ -1480,7 +1473,7 @@ class ParseTest(unittest.TestCase):
 		node = parsers.parseString("<z>gurk&amp;hurz&#42;hinz&#x666;hunz</z>", saxparser=parsers.ExpatParser)
 		self.assertEqual(len(node), 1)
 		self.assertEqual(len(node[0]), 1)
-		self.assertEqual(node[0][0].startloc.getSystemId(), "root:STRING")
+		self.assertEqual(node[0][0].startloc.getSystemId(), "STRING")
 		self.assertEqual(node[0][0].startloc.getLineNumber(), 1)
 		self.assertEqual(node[0][0].startloc.getColumnNumber(), 3)
 
@@ -1525,8 +1518,8 @@ class ParseTest(unittest.TestCase):
 		node = parsers.parseString('<Test required="foo"/>', prefixes=prefixes)
 		self.assertEqual(str(node[0]["required"]), "foo")
 
-		warnings.filterwarnings("error", category=errors.RequiredAttrMissingWarning)
-		self.assertSAXRaises(errors.RequiredAttrMissingWarning, parsers.parseString, '<Test/>', prefixes=prefixes)
+		warnings.filterwarnings("error", category=xsc.RequiredAttrMissingWarning)
+		self.assertSAXRaises(xsc.RequiredAttrMissingWarning, parsers.parseString, '<Test/>', prefixes=prefixes)
 
 	def test_parsevalueattrs(self):
 		# Parser should complain about attributes with illegal values, when a set of values is specified
@@ -1537,10 +1530,10 @@ class ParseTest(unittest.TestCase):
 
 		prefixes = xsc.Prefixes(__ns__)
 
-		warnings.filterwarnings("error", category=errors.IllegalAttrValueWarning)
+		warnings.filterwarnings("error", category=xsc.IllegalAttrValueWarning)
 		node = parsers.parseString('<Test withvalues="bar"/>', prefixes=prefixes)
 		self.assertEqual(str(node[0]["withvalues"]), "bar")
-		self.assertSAXRaises(errors.IllegalAttrValueWarning, parsers.parseString, '<Test withvalues="baz"/>', prefixes=prefixes)
+		self.assertSAXRaises(xsc.IllegalAttrValueWarning, parsers.parseString, '<Test withvalues="baz"/>', prefixes=prefixes)
 
 	class __ns__(xsc.Namespace):
 		xmlname = "foo"
@@ -1562,13 +1555,13 @@ class ParseTest(unittest.TestCase):
 
 	def check_parsestrictentities(self, source, result, parserfactory):
 		# in the strict parser the errors will always be raised, so change them into errors to verify that
-		warnings.filterwarnings("error", category=errors.MalformedCharRefWarning)
+		warnings.filterwarnings("error", category=xsc.MalformedCharRefWarning)
 
 		prefixes = xsc.Prefixes([self.__class__.__ns__, chars])
 		self.check_parseentities(source, result, prefixes=prefixes, saxparser=parserfactory)
 		for bad in ("&", "&#x", "&&", "&#x;", "&#fg;", "&#999999999;", "&#;", "&#y;", "&#x;", "&#xy;"):
-			self.assertSAXRaises((errors.MalformedCharRefWarning, expat.ExpatError), self.check_parseentities, bad, u"", prefixes=prefixes, saxparser=parserfactory)
-		self.assertSAXRaises(errors.IllegalEntityError, self.check_parseentities, "&baz;", u"", prefixes=prefixes, saxparser=parserfactory)
+			self.assertSAXRaises((xsc.MalformedCharRefWarning, expat.ExpatError), self.check_parseentities, bad, u"", prefixes=prefixes, saxparser=parserfactory)
+		self.assertSAXRaises(xsc.IllegalEntityError, self.check_parseentities, "&baz;", u"", prefixes=prefixes, saxparser=parserfactory)
 
 	def test_parsestrictentities_sgmlop(self):
 		self.check_parsestrictentities(
@@ -1585,7 +1578,7 @@ class ParseTest(unittest.TestCase):
 		)
 
 	def check_parsebadentities(self, parserfactory):
-		warnings.filterwarnings("ignore", category=errors.MalformedCharRefWarning)
+		warnings.filterwarnings("ignore", category=xsc.MalformedCharRefWarning)
 
 		prefixes = xsc.Prefixes([self.__class__.__ns__, chars])
 		tests = [
@@ -1630,7 +1623,7 @@ class ParseTest(unittest.TestCase):
 	def test_sysid(self):
 		# Default system ids and explicitely specified system ids should end up in the location info of the resulting XML tree
 		node = parsers.parseString("gurk")
-		self.assertEqual(node[0].startloc.sysid, "root:STRING")
+		self.assertEqual(node[0].startloc.sysid, "STRING")
 
 		node = parsers.parseString("gurk", base="root:gurk.xmlxsc")
 		self.assertEqual(node[0].startloc.sysid, "root:gurk.xmlxsc")
@@ -1715,7 +1708,7 @@ class DTD2XSCTest(unittest.TestCase):
 		ns = self.dtd2ns(dtdstring, "foo")
 
 		self.assert_(issubclass(ns, xsc.Namespace))
-		self.assertEqual(ns.xmlname, ("__ns__", "foo"))
+		self.assertEqual(ns.xmlname, "foo")
 		self.assertEqual(ns.xmlurl, "http://xmlns.foo.com/foo")
 		self.assert_(isinstance(ns.foo.model, sims.Elements))
 		self.assertEqual(len(ns.foo.model.elements), 1)
@@ -1738,15 +1731,15 @@ class DTD2XSCTest(unittest.TestCase):
 
 		# Attributes are alphabetically sorted
 		self.assert_(issubclass(ns.bar.Attrs.bar_4, xsc.TextAttr))
-		self.assertEqual(ns.bar.Attrs.bar_4.xmlname, ("bar_4", "bar-4"))
+		self.assertEqual(ns.bar.Attrs.bar_4.xmlname, "bar-4")
 		self.assertEqual(ns.bar.Attrs.bar_4.values, ("bar-4a", "bar-4b"))
 
 		self.assert_(issubclass(ns.bar.Attrs.bar_42, xsc.TextAttr))
-		self.assertEqual(ns.bar.Attrs.bar_42.xmlname, ("bar_42", "bar_4"))
+		self.assertEqual(ns.bar.Attrs.bar_42.xmlname, "bar_4")
 		self.assertEqual(ns.bar.Attrs.bar_42.values, ("bar_4a", "bar_4b"))
 
 		self.assert_(issubclass(ns.bar.Attrs.bar_422, xsc.TextAttr))
-		self.assertEqual(ns.bar.Attrs.bar_422.xmlname, ("bar_422", "bar_42"))
+		self.assertEqual(ns.bar.Attrs.bar_422.xmlname, "bar_42")
 		self.assertEqual(ns.bar.Attrs.bar_422.values, ("bar_42a", "bar_42b"))
 
 	def test_charref(self):
@@ -1767,7 +1760,8 @@ class DTD2XSCTest(unittest.TestCase):
 		"""
 		ns = self.dtd2ns(dtdstring, "foo")
 		self.assert_(issubclass(ns.foo.Attrs.class_, xsc.TextAttr))
-		self.assertEqual(ns.foo.Attrs.class_.xmlname, ("class_", "class"))
+		self.assertEqual(ns.foo.Attrs.class_.__name__, "class_")
+		self.assertEqual(ns.foo.Attrs.class_.xmlname, u"class")
 
 	def test_quotes(self):
 		dtdstring = """<?xml version='1.0' encoding='us-ascii'?>
@@ -1909,8 +1903,8 @@ class TLD2XSCTest(unittest.TestCase):
 		</taglib>
 		"""
 		ns = self.tld2ns(tldstring, "foo")
-		self.assertEqual(ns.xmlname, ("__ns__", "foo"))
-		self.assertEqual(ns.bar.xmlname, ("bar", "bar"))
+		self.assertEqual(ns.xmlname, u"foo")
+		self.assertEqual(ns.bar.xmlname, u"bar")
 		self.assert_(isinstance(ns.bar.model, sims.Empty))
 		self.assertEqual(ns.bar.__doc__.strip(), "info")
 
@@ -2380,6 +2374,123 @@ class XFindTestItemSlice(unittest.TestCase):
 		self.checkids(ds[0]/html.div[1:-1]/html.div[-1:], "6")
 
 
+class PickleTest(unittest.TestCase):
+	def test_pickle(self):
+		e = xsc.Frag(
+			xml.XML10(),
+			html.DocTypeXHTML10transitional(),
+			xsc.Comment("foo"),
+			html.html(xml.Attrs(lang="de"), lang="de"),
+			php.expression("$foo"),
+			chars.nbsp(),
+			abbr.xml(),
+		)
+		e.append(e[3])
+		e2 = cPickle.loads(cPickle.dumps(e))
+		self.assertEqual(e, e2)
+		self.assert_(e2[3] is e2[-1])
+
+
+class WalkTest(unittest.TestCase):
+	def node2str(self, node):
+		if isinstance(node, xsc.Node):
+			if isinstance(node, xsc.Text):
+				return "#"
+			else:
+				return node.xmlname
+		else:
+			return ".".join(map(self.node2str, node))
+
+	def check_walk(self, node, filter, result, inmode=xsc.walknode, outmode=xsc.walknode):
+		self.assertEqual(map(self.node2str, node.walk(filter, inmode=inmode, outmode=outmode)), result)
+
+	def test_walk1(self):
+		node = createfrag()
+		def filter(*args):
+			return (True, xsc.enterattrs, xsc.entercontent, True)
+
+		modes = (xsc.walknode, xsc.walkpath, xsc.walkindex, xsc.walkrootindex)
+		for inmode in modes:
+			for outmode in modes:
+				list(node.walk((True, xsc.entercontent, True), inmode=inmode, outmode=outmode))
+				list(node.walk(filter, inmode=inmode, outmode=outmode))
+
+	def test_walk2(self):
+		node = html.div(html.tr(html.th("gurk"), html.td("hurz"), id=html.b(42)), class_=html.i("hinz"))
+
+		def filtertopdown(node):
+			return (isinstance(node, xsc.Element), xsc.entercontent)
+		def filterbottomup(node):
+			return (xsc.entercontent, isinstance(node, xsc.Element))
+		def filtertopdownattrs(node):
+			return (isinstance(node, xsc.Element), xsc.enterattrs, xsc.entercontent)
+		def filterbottomupattrs(node):
+			return (xsc.enterattrs, xsc.entercontent, isinstance(node, xsc.Element))
+		def filtertopdowntextonlyinattr(path):
+			for node in path:
+				if isinstance(node, xsc.Attr):
+					inattr = True
+					break
+			else:
+				inattr = False
+			node = path[-1]
+			if isinstance(node, xsc.Element):
+				return (True, xsc.enterattrs, xsc.entercontent)
+			if inattr and isinstance(node, xsc.Text):
+				return (True, )
+			else:
+				return (xsc.entercontent, )
+
+		def filtertopdownattrwithoutcontent(node):
+			if isinstance(node, xsc.Element):
+				return (True, xsc.entercontent, xsc.enterattrs)
+			elif isinstance(node, (xsc.Attr, xsc.Text)):
+				return (True, )
+			else:
+				return (xsc.entercontent, )
+
+		self.check_walk(node, filtertopdown, ["div", "tr", "th", "td"])
+		self.check_walk(node, filterbottomup, ["th", "td", "tr", "div"])
+		self.check_walk(node, filtertopdownattrs, ["div", "i", "tr", "b", "th", "td"])
+		self.check_walk(node, filtertopdownattrs, ["div", "div.class.i", "div.tr", "div.tr.id.b", "div.tr.th", "div.tr.td"], outmode=xsc.walkpath)
+		self.check_walk(node, filterbottomupattrs, ["div.class.i", "div.tr.id.b", "div.tr.th", "div.tr.td", "div.tr", "div"], outmode=xsc.walkpath)
+		self.check_walk(node, filtertopdowntextonlyinattr, ["div", "div.class.i", "div.class.i.#", "div.tr", "div.tr.id.b", "div.tr.id.b.#", "div.tr.th", "div.tr.td"], inmode=xsc.walkpath, outmode=xsc.walkpath)
+		self.check_walk(node, filtertopdownattrwithoutcontent, ["div", "div.tr", "div.tr.th", "div.tr.th.#", "div.tr.td", "div.tr.td.#", "div.tr.id", "div.class"], outmode=xsc.walkpath)
+
+	def test_walkindex(self):
+		e = html.div(
+			"foo",
+			html.a(
+				"bar",
+				xml.Attrs(lang="en"),
+				href="baz",
+			),
+			"gurk",
+		)
+		res = list(e.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkindex))
+		exp = [
+			[0],
+			[1, 0],
+			[1, "href", 0],
+			[1, (xml, "lang"), 0], # FIXME: This depends on dictionary iteration order
+			[2]
+		]
+		self.assertEqual(res, exp)
+
+	def test_walkindexisnode(self):
+		# Check that all walk modes return the same data
+		for node in allnodes():
+			l1 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walknode))
+			l2 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkpath))
+			l3 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkindex))
+			l4 = list(node.walk(xsc.FindTypeAllAttrs(xsc.Text), outmode=xsc.walkrootindex))
+			self.assert_(len(l1) == len(l2) == len(l3) == len(l4))
+			for (subnode, path, index, (root, rindex)) in zip(l1, l2, l3, l4):
+				self.assert_(subnode is path[-1])
+				self.assert_(subnode is node[index])
+				self.assert_(subnode is root[rindex])
+
+
 def test_main():
 	unittest.main()
 
